# Swiss Ephemeris API Deployment Guide for orbo.site

This guide will help you deploy the Swiss Ephemeris API to your domain: www.orbo.site

## Prerequisites

1. A web server with Python 3.8+ installed
2. SSH access to your server
3. Domain configured to point to your server

## Deployment Steps

### 1. Prepare the Files

You have two files to deploy:
- `ephemeris_data.py` - Contains the encoded ephemeris files
- `astro_api_server.py` - The main API server script

### 2. Upload to Your Server

Using SFTP or SCP, upload both files to your server:

```bash
# Example using scp
scp ephemeris_data.py user@orbo.site:/path/to/deployment/
scp astro_api_server.py user@orbo.site:/path/to/deployment/
```

### 3. Install Dependencies on the Server

SSH into your server and install the required packages:

```bash
ssh user@orbo.site
cd /path/to/deployment/
pip install fastapi uvicorn pyswisseph pydantic
```

### 4. Test the API

Run the API server to make sure it works:

```bash
python astro_api_server.py --port 8000
```

Visit http://localhost:8000/docs on your server to ensure the API is working correctly.

### 5. Set Up a Production Environment

For production, you'll want to run the API as a service. Here's how to do it with systemd:

Create a service file at `/etc/systemd/system/astro-api.service`:

```ini
[Unit]
Description=Swiss Ephemeris Astrology API
After=network.target

[Service]
User=your_username
WorkingDirectory=/path/to/deployment
ExecStart=/usr/bin/python3 astro_api_server.py --port 8000 --host 127.0.0.1 --api-key your_secure_api_key
Restart=on-failure
RestartSec=5s

[Install]
WantedBy=multi-user.target
```

Enable and start the service:

```bash
sudo systemctl daemon-reload
sudo systemctl enable astro-api
sudo systemctl start astro-api
```

### 6. Set Up a Reverse Proxy

For your www.orbo.site domain, you'll need to set up a reverse proxy. Here's an example using Nginx:

```nginx
server {
    server_name www.orbo.site orbo.site;
    
    location / {
        proxy_pass http://127.0.0.1:8000;
        proxy_set_header Host $host;
        proxy_set_header X-Real-IP $remote_addr;
        proxy_set_header X-Forwarded-For $proxy_add_x_forwarded_for;
        proxy_set_header X-Forwarded-Proto $scheme;
    }
    
    listen 80;
}
```

### 7. Set Up SSL

Use Let's Encrypt to secure your API with SSL:

```bash
sudo apt install certbot python3-certbot-nginx
sudo certbot --nginx -d orbo.site -d www.orbo.site
```

### 8. Test the Deployed API

Visit https://www.orbo.site/docs to ensure the API is working correctly.

## Using the API with LLMs

To make your API accessible to LLMs:

1. Use the API key for authentication:
   ```
   curl -H "X-API-Key: your_api_key" https://www.orbo.site/planets?date=2025-05-13
   ```

2. Create an LLM tool definition that calls your API endpoints.

3. Document the JSON format of your API responses for LLMs to parse.

## Example LLM Tool Definition

```json
{
  "type": "function",
  "function": {
    "name": "get_natal_chart",
    "description": "Get astrological natal chart for a specific date, time and location",
    "parameters": {
      "type": "object",
      "properties": {
        "date": {
          "type": "string",
          "description": "Date in YYYY-MM-DD format"
        },
        "time": {
          "type": "string",
          "description": "Time in HH:MM:SS format"
        },
        "latitude": {
          "type": "number",
          "description": "Geographical latitude in degrees (-90 to 90)"
        },
        "longitude": {
          "type": "number",
          "description": "Geographical longitude in degrees (-180 to 180)"
        }
      },
      "required": ["date", "time", "latitude", "longitude"]
    }
  }
}
```

## Monitoring and Maintenance

1. **Check logs**: `sudo journalctl -u astro-api`
2. **Restart service**: `sudo systemctl restart astro-api`
3. **Update API**: Replace the files and restart the service

## Support

If you need help with deployment, contact the developer or consider hiring a DevOps specialist to assist with the setup.
