from fastapi import FastAPI, HTTPException, Query, Depends
from fastapi.middleware.cors import CORSMiddleware
from typing import Dict, List, Optional, Any, Union
from datetime import datetime
import os
import swisseph as swe
from dotenv import load_dotenv
import logging

from .models import (
    PlanetPosition, 
    PlanetType, 
    HouseSystem, 
    HouseCusps, 
    AspectType, 
    Aspect, 
    TransitInfo, 
    NatalChart
)
from .utils import (
    set_ephemeris_path, 
    julian_day_ut, 
    parse_datetime, 
    calculate_planet_position,
    get_planet_id, 
    calculate_houses, 
    calculate_aspects, 
    is_retrograde
)

# Load environment variables
load_dotenv()

# Set up logging
logging.basicConfig(
    level=logging.INFO,
    format='%(asctime)s - %(name)s - %(levelname)s - %(message)s'
)
logger = logging.getLogger(__name__)

# Initialize FastAPI app
app = FastAPI(
    title="Swiss Ephemeris API",
    description="API for astrological calculations using Swiss Ephemeris",
    version="1.0.0",
    docs_url="/docs",
    redoc_url="/redoc",
)

# Add CORS middleware
app.add_middleware(
    CORSMiddleware,
    allow_origins=["*"],  # Change this in production to specific origins
    allow_credentials=True,
    allow_methods=["*"],
    allow_headers=["*"],
)

# Set ephemeris path at startup
@app.on_event("startup")
async def startup_event():
    try:
        # Get path from environment variable or use default
        ephe_path = os.environ.get("EPHE_PATH", r"C:\Users\hunta\Documents\GitHub\swisseph\ephe")
        set_ephemeris_path(ephe_path)
        logger.info(f"Swiss Ephemeris path set to: {ephe_path}")
    except Exception as e:
        logger.error(f"Failed to set ephemeris path: {str(e)}")
        raise


@app.get("/")
async def root():
    """
    Root endpoint that returns a welcome message and basic API info
    """
    return {
        "message": "Welcome to the Swiss Ephemeris API", 
        "docs": "/docs",
        "version": "1.0.0"
    }


@app.get("/planet/{planet}", response_model=PlanetPosition, tags=["Planets"])
async def get_planet_position(
    planet: PlanetType,
    date: str = Query(..., description="Date in YYYY-MM-DD format"),
    time: Optional[str] = Query("00:00:00", description="Time in HH:MM:SS format"),
    geo_lat: Optional[float] = Query(0.0, description="Geographical latitude (-90 to +90)"),
    geo_lon: Optional[float] = Query(0.0, description="Geographical longitude (-180 to +180)"),
    altitude: Optional[float] = Query(0.0, description="Altitude in meters"),
    apparent: Optional[bool] = Query(True, description="Whether to calculate apparent position")
):
    """
    Get the position of a planet at a specific date and time.
    
    - **planet**: Planet name (sun, moon, mercury, etc.)
    - **date**: Date in YYYY-MM-DD format
    - **time**: Time in HH:MM:SS format (default: 00:00:00)
    - **geo_lat**: Geographical latitude (-90 to +90, default: 0.0)
    - **geo_lon**: Geographical longitude (-180 to +180, default: 0.0)
    - **altitude**: Altitude above sea level in meters (default: 0.0)
    - **apparent**: Whether to calculate apparent position (true) or true position (false)
    """
    try:
        # Parse date and time
        year, month, day, hour = parse_datetime(date, time)
        
        # Convert to Julian day
        jd = julian_day_ut(year, month, day, hour)
        
        # Set calculation flags
        flags = swe.FLG_SWIEPH
        if apparent:
            flags |= swe.FLG_TRUEPOS
        else:
            flags |= swe.FLG_SWIEPH
        
        # Calculate planet position
        planet_id = get_planet_id(planet)
        xx, ret = swe.calc_ut(jd, planet_id, flags)
        
        # Return planet position
        return PlanetPosition(
            longitude=xx[0],
            latitude=xx[1],
            distance=xx[2],
            longitude_speed=xx[3],
            latitude_speed=xx[4],
            distance_speed=xx[5]
        )
    except Exception as e:
        logger.error(f"Error calculating planet position: {str(e)}")
        raise HTTPException(status_code=400, detail=str(e))


@app.get("/planets", response_model=Dict[str, PlanetPosition], tags=["Planets"])
async def get_all_planets(
    date: str = Query(..., description="Date in YYYY-MM-DD format"),
    time: Optional[str] = Query("00:00:00", description="Time in HH:MM:SS format"),
    geo_lat: Optional[float] = Query(0.0, description="Geographical latitude (-90 to +90)"),
    geo_lon: Optional[float] = Query(0.0, description="Geographical longitude (-180 to +180)"),
    altitude: Optional[float] = Query(0.0, description="Altitude in meters"),
    apparent: Optional[bool] = Query(True, description="Whether to calculate apparent position")
):
    """
    Get the positions of all major planets at a specific date and time.
    
    - **date**: Date in YYYY-MM-DD format
    - **time**: Time in HH:MM:SS format (default: 00:00:00)
    - **geo_lat**: Geographical latitude (-90 to +90, default: 0.0)
    - **geo_lon**: Geographical longitude (-180 to +180, default: 0.0)
    - **altitude**: Altitude above sea level in meters (default: 0.0)
    - **apparent**: Whether to calculate apparent position (true) or true position (false)
    """
    try:
        # Parse date and time
        year, month, day, hour = parse_datetime(date, time)
        
        # Convert to Julian day
        jd = julian_day_ut(year, month, day, hour)
        
        # Set calculation flags
        flags = swe.FLG_SWIEPH
        if apparent:
            flags |= swe.FLG_TRUEPOS
        else:
            flags |= swe.FLG_SWIEPH
        
        # Get all planets
        result = {}
        for planet in PlanetType:
            planet_id = get_planet_id(planet)
            xx, ret = swe.calc_ut(jd, planet_id, flags)
            result[planet.value] = PlanetPosition(
                longitude=xx[0],
                latitude=xx[1],
                distance=xx[2],
                longitude_speed=xx[3],
                latitude_speed=xx[4],
                distance_speed=xx[5]
            )
        
        return result
    except Exception as e:
        logger.error(f"Error calculating planet positions: {str(e)}")
        raise HTTPException(status_code=400, detail=str(e))


@app.get("/houses", response_model=HouseCusps, tags=["Houses"])
async def get_houses(
    date: str = Query(..., description="Date in YYYY-MM-DD format"),
    time: Optional[str] = Query("00:00:00", description="Time in HH:MM:SS format"),
    geo_lat: float = Query(..., description="Geographical latitude (-90 to +90)"),
    geo_lon: float = Query(..., description="Geographical longitude (-180 to +180)"),
    house_system: HouseSystem = Query(HouseSystem.PLACIDUS, description="House system to use")
):
    """
    Calculate house cusps for a specific date, time, and location.
    
    - **date**: Date in YYYY-MM-DD format
    - **time**: Time in HH:MM:SS format (default: 00:00:00)
    - **geo_lat**: Geographical latitude (-90 to +90)
    - **geo_lon**: Geographical longitude (-180 to +180)
    - **house_system**: House system to use (default: Placidus)
    """
    try:
        # Parse date and time
        year, month, day, hour = parse_datetime(date, time)
        
        # Convert to Julian day
        jd = julian_day_ut(year, month, day, hour)
        
        # Calculate houses
        houses = calculate_houses(jd, geo_lat, geo_lon, house_system.value)
        
        return HouseCusps(**houses)
    except Exception as e:
        logger.error(f"Error calculating houses: {str(e)}")
        raise HTTPException(status_code=400, detail=str(e))


@app.get("/aspects", response_model=List[Aspect], tags=["Aspects"])
async def get_aspects(
    date: str = Query(..., description="Date in YYYY-MM-DD format"),
    time: Optional[str] = Query("00:00:00", description="Time in HH:MM:SS format"),
    orb: Optional[float] = Query(8.0, description="Maximum orb in degrees"),
    include_planets: List[PlanetType] = Query(
        [
            PlanetType.SUN, PlanetType.MOON, PlanetType.MERCURY, 
            PlanetType.VENUS, PlanetType.MARS, PlanetType.JUPITER, 
            PlanetType.SATURN, PlanetType.URANUS, PlanetType.NEPTUNE, 
            PlanetType.PLUTO
        ],
        description="Planets to include in aspect calculations"
    )
):
    """
    Calculate aspects between planets for a specific date and time.
    
    - **date**: Date in YYYY-MM-DD format
    - **time**: Time in HH:MM:SS format (default: 00:00:00)
    - **orb**: Maximum orb in degrees (default: 8.0)
    - **include_planets**: List of planets to include in aspect calculations
    """
    try:
        # Parse date and time
        year, month, day, hour = parse_datetime(date, time)
        
        # Convert to Julian day
        jd = julian_day_ut(year, month, day, hour)
        
        # Calculate planet positions
        positions = {}
        for planet in include_planets:
            planet_id = get_planet_id(planet)
            positions[planet] = calculate_planet_position(jd, planet_id)
        
        # Calculate aspects
        aspects = calculate_aspects(positions, orb)
        
        return aspects
    except Exception as e:
        logger.error(f"Error calculating aspects: {str(e)}")
        raise HTTPException(status_code=400, detail=str(e))


@app.get("/transits", response_model=List[TransitInfo], tags=["Transits"])
async def get_transits(
    birth_date: str = Query(..., description="Birth date in YYYY-MM-DD format"),
    birth_time: Optional[str] = Query("00:00:00", description="Birth time in HH:MM:SS format"),
    transit_date: str = Query(..., description="Transit date in YYYY-MM-DD format"),
    transit_time: Optional[str] = Query("00:00:00", description="Transit time in HH:MM:SS format"),
    orb: Optional[float] = Query(1.0, description="Maximum orb in degrees"),
    include_planets: List[PlanetType] = Query(
        [
            PlanetType.SUN, PlanetType.MOON, PlanetType.MERCURY, 
            PlanetType.VENUS, PlanetType.MARS, PlanetType.JUPITER, 
            PlanetType.SATURN, PlanetType.URANUS, PlanetType.NEPTUNE, 
            PlanetType.PLUTO
        ],
        description="Planets to include in transit calculations"
    )
):
    """
    Calculate transits to natal planets for a specific date and time.
    
    - **birth_date**: Birth date in YYYY-MM-DD format
    - **birth_time**: Birth time in HH:MM:SS format (default: 00:00:00)
    - **transit_date**: Transit date in YYYY-MM-DD format
    - **transit_time**: Transit time in HH:MM:SS format (default: 00:00:00)
    - **orb**: Maximum orb in degrees (default: 1.0)
    - **include_planets**: List of planets to include in transit calculations
    """
    try:
        # Parse birth date and time
        birth_year, birth_month, birth_day, birth_hour = parse_datetime(birth_date, birth_time)
        
        # Parse transit date and time
        transit_year, transit_month, transit_day, transit_hour = parse_datetime(transit_date, transit_time)
        
        # Convert to Julian days
        birth_jd = julian_day_ut(birth_year, birth_month, birth_day, birth_hour)
        transit_jd = julian_day_ut(transit_year, transit_month, transit_day, transit_hour)
        
        # Calculate natal planet positions
        natal_positions = {}
        for planet in include_planets:
            planet_id = get_planet_id(planet)
            natal_positions[planet] = calculate_planet_position(birth_jd, planet_id)
        
        # Calculate transit planet positions
        transit_positions = {}
        for planet in include_planets:
            planet_id = get_planet_id(planet)
            transit_positions[planet] = calculate_planet_position(transit_jd, planet_id)
        
        # Calculate transits
        transits = []
        for transit_planet in include_planets:
            transit_pos = transit_positions[transit_planet]["longitude"]
            transit_speed = transit_positions[transit_planet]["longitude_speed"]
            
            for natal_planet in include_planets:
                natal_pos = natal_positions[natal_planet]["longitude"]
                
                # Calculate angle between transit and natal planet
                angle = (transit_pos - natal_pos) % 360
                
                # Get aspect type
                from .utils import get_aspect_type, calculate_orb
                aspect_type = get_aspect_type(angle, orb)
                
                # Create transit info
                transit_info = TransitInfo(
                    transit_date=transit_date,
                    planet=transit_planet,
                    natal_position=natal_pos,
                    transit_position=transit_pos,
                    is_retrograde=is_retrograde(transit_speed)
                )
                
                # Add aspect info if there is an aspect
                if aspect_type:
                    from .utils import get_aspect_angle
                    exact_angle = get_aspect_angle(aspect_type)
                    orb_value = calculate_orb(angle, exact_angle)
                    
                    transit_info.aspect_angle = angle
                    transit_info.aspect_type = aspect_type
                    transit_info.orb = orb_value
                
                transits.append(transit_info)
        
        return transits
    except Exception as e:
        logger.error(f"Error calculating transits: {str(e)}")
        raise HTTPException(status_code=400, detail=str(e))


@app.get("/chart", response_model=NatalChart, tags=["Charts"])
async def get_natal_chart(
    date: str = Query(..., description="Date in YYYY-MM-DD format"),
    time: Optional[str] = Query("00:00:00", description="Time in HH:MM:SS format"),
    geo_lat: float = Query(..., description="Geographical latitude (-90 to +90)"),
    geo_lon: float = Query(..., description="Geographical longitude (-180 to +180)"),
    house_system: HouseSystem = Query(HouseSystem.PLACIDUS, description="House system to use"),
    orb: Optional[float] = Query(8.0, description="Maximum orb for aspects in degrees"),
    include_planets: List[PlanetType] = Query(
        [
            PlanetType.SUN, PlanetType.MOON, PlanetType.MERCURY, 
            PlanetType.VENUS, PlanetType.MARS, PlanetType.JUPITER, 
            PlanetType.SATURN, PlanetType.URANUS, PlanetType.NEPTUNE, 
            PlanetType.PLUTO
        ],
        description="Planets to include in chart"
    )
):
    """
    Calculate a complete natal chart for a specific date, time, and location.
    
    - **date**: Date in YYYY-MM-DD format
    - **time**: Time in HH:MM:SS format (default: 00:00:00)
    - **geo_lat**: Geographical latitude (-90 to +90)
    - **geo_lon**: Geographical longitude (-180 to +180)
    - **house_system**: House system to use (default: Placidus)
    - **orb**: Maximum orb for aspects in degrees (default: 8.0)
    - **include_planets**: List of planets to include in chart
    """
    try:
        # Parse date and time
        year, month, day, hour = parse_datetime(date, time)
        
        # Convert to Julian day
        jd = julian_day_ut(year, month, day, hour)
        
        # Calculate planet positions
        positions = {}
        for planet in include_planets:
            planet_id = get_planet_id(planet)
            positions[planet] = calculate_planet_position(jd, planet_id)
        
        # Calculate houses
        houses = calculate_houses(jd, geo_lat, geo_lon, house_system.value)
        
        # Calculate aspects
        aspects = calculate_aspects(positions, orb)
        
        # Format planet positions for the response
        planets_dict = {planet.value: PlanetPosition(**pos) for planet, pos in positions.items()}
        
        # Create and return the natal chart
        return NatalChart(
            planets=planets_dict,
            houses=HouseCusps(**houses),
            aspects=aspects
        )
    except Exception as e:
        logger.error(f"Error calculating natal chart: {str(e)}")
        raise HTTPException(status_code=400, detail=str(e))


@app.get("/health", tags=["System"])
async def health_check():
    """
    Health check endpoint to verify the API is running properly
    """
    try:
        # Try a simple calculation to check if Swiss Ephemeris is working
        jd = swe.julday(2000, 1, 1, 12)
        xx, ret = swe.calc_ut(jd, swe.SUN)
        
        return {
            "status": "ok",
            "swiss_ephemeris": "ok",
            "version": app.version
        }
    except Exception as e:
        logger.error(f"Health check failed: {str(e)}")
        return {
            "status": "error",
            "message": str(e),
            "version": app.version
        }


# Error handling
@app.exception_handler(HTTPException)
async def http_exception_handler(request, exc):
    return {"status": "error", "code": exc.status_code, "message": exc.detail}


@app.exception_handler(Exception)
async def general_exception_handler(request, exc):
    logger.error(f"Unhandled exception: {str(exc)}")
    return {"status": "error", "message": str(exc)}
