from pydantic import BaseModel, Field
from typing import List, Optional, Dict, Any, Union
from enum import Enum


class PlanetPosition(BaseModel):
    """Model for planet positions"""
    longitude: float = Field(..., description="Longitude in degrees")
    latitude: float = Field(..., description="Latitude in degrees")
    distance: float = Field(..., description="Distance in AU")
    longitude_speed: float = Field(..., description="Daily speed in longitude (degrees/day)")
    latitude_speed: float = Field(..., description="Daily speed in latitude (degrees/day)")
    distance_speed: float = Field(..., description="Daily speed in distance (AU/day)")


class PlanetType(str, Enum):
    """Enumeration of planet types for clearer API interface"""
    SUN = "sun"  # SE_SUN = 0
    MOON = "moon"  # SE_MOON = 1
    MERCURY = "mercury"  # SE_MERCURY = 2
    VENUS = "venus"  # SE_VENUS = 3
    MARS = "mars"  # SE_MARS = 4
    JUPITER = "jupiter"  # SE_JUPITER = 5
    SATURN = "saturn"  # SE_SATURN = 6
    URANUS = "uranus"  # SE_URANUS = 7
    NEPTUNE = "neptune"  # SE_NEPTUNE = 8
    PLUTO = "pluto"  # SE_PLUTO = 9
    MEAN_NODE = "mean_node"  # SE_MEAN_NODE = 10
    TRUE_NODE = "true_node"  # SE_TRUE_NODE = 11
    MEAN_APOG = "mean_apogee"  # SE_MEAN_APOG = 12
    OSCU_APOG = "osculating_apogee"  # SE_OSCU_APOG = 13
    EARTH = "earth"  # SE_EARTH = 14
    CHIRON = "chiron"  # SE_CHIRON = 15
    PHOLUS = "pholus"  # SE_PHOLUS = 16
    CERES = "ceres"  # SE_CERES = 17
    PALLAS = "pallas"  # SE_PALLAS = 18
    JUNO = "juno"  # SE_JUNO = 19
    VESTA = "vesta"  # SE_VESTA = 20


class HouseSystem(str, Enum):
    """Enumeration of house systems"""
    PLACIDUS = "P"
    KOCH = "K"
    PORPHYRIUS = "O"
    REGIOMONTANUS = "R"
    CAMPANUS = "C"
    EQUAL = "E"
    WHOLE_SIGN = "W"
    MERIDIAN = "X"
    MORINUS = "M"
    KRUSINSKI = "U"
    ALCABITIUS = "H"
    POLICH_PAGE = "T"
    EQUAL_MC = "B"
    EQUAL_ARMC = "Y"
    VEHLOW_EQUAL = "V"
    AXIAL_ROTATION = "A"
    HORIZONTAL = "G"
    SUNSHINE = "I"
    SUNSHINE_TREINDL = "i"
    GAUQUELIN = "L"


class HouseCusps(BaseModel):
    """Model for house cusps"""
    house_cusps: List[float] = Field(..., description="House cusps in degrees")
    ascendant: float = Field(..., description="Ascendant in degrees")
    mc: float = Field(..., description="Medium Coeli (MC) in degrees")
    armc: float = Field(..., description="ARMC in degrees")
    vertex: float = Field(..., description="Vertex in degrees")
    equatorial_ascendant: float = Field(..., description="Equatorial Ascendant in degrees")
    co_ascendant_koch: float = Field(..., description="Co-Ascendant (Koch) in degrees")
    co_ascendant_munkasey: float = Field(..., description="Co-Ascendant (Munkasey) in degrees")
    polar_ascendant: float = Field(..., description="Polar Ascendant in degrees")


class AspectType(str, Enum):
    """Enumeration of aspect types"""
    CONJUNCTION = "conjunction"
    SEXTILE = "sextile"
    SQUARE = "square"
    TRINE = "trine"
    OPPOSITION = "opposition"
    QUINCUNX = "quincunx"
    SEMISEXTILE = "semisextile"
    SEMISQUARE = "semisquare"
    SESQUIQUADRATE = "sesquiquadrate"
    QUINTILE = "quintile"
    BIQUINTILE = "biquintile"


class Aspect(BaseModel):
    """Model for aspects between planets"""
    planet1: PlanetType
    planet2: PlanetType
    aspect_type: AspectType
    angle: float = Field(..., description="Exact angle of the aspect in degrees")
    orb: float = Field(..., description="Orb (deviation from exact aspect) in degrees")
    is_applying: bool = Field(..., description="True if the aspect is applying, False if separating")


class TransitInfo(BaseModel):
    """Model for transit information"""
    transit_date: str
    planet: PlanetType
    natal_position: float = Field(..., description="Natal position in degrees")
    transit_position: float = Field(..., description="Transit position in degrees")
    aspect_angle: Optional[float] = Field(None, description="Aspect angle if any")
    aspect_type: Optional[AspectType] = Field(None, description="Type of aspect if any")
    orb: Optional[float] = Field(None, description="Orb of aspect if any")
    is_retrograde: bool = Field(..., description="Whether the transiting planet is retrograde")


class NatalChart(BaseModel):
    """Model for natal chart data"""
    planets: Dict[str, PlanetPosition]
    houses: HouseCusps
    aspects: List[Aspect] = []
