import swisseph as swe
import os
from datetime import datetime
from typing import Dict, List, Optional, Tuple, Any, Union

from .models import PlanetType, AspectType, Aspect, HouseSystem


def set_ephemeris_path(path: Optional[str] = None) -> None:
    """Set the path to the Swiss Ephemeris files"""
    if path:
        ephe_path = path
    else:
        # Use the path from environment variable or default
        ephe_path = os.environ.get("EPHE_PATH", r"C:\Users\hunta\Documents\GitHub\swisseph\ephe")
    
    swe.set_ephe_path(ephe_path)
    
    # Log the ephemeris path for verification
    print(f"Ephemeris path set to: {ephe_path}")
    
    # Verify that the main ephemeris files can be found
    try:
        jd = swe.julday(2000, 1, 1, 12)
        pos, flags = swe.calc_ut(jd, swe.SUN)
        print("Successfully found ephemeris files")
    except Exception as e:
        print(f"Error: Failed to calculate position - {str(e)}")
        print("This might indicate that the ephemeris files cannot be found")
        
    # Additional suggestion for JPL files
    if os.path.exists(os.path.join(ephe_path, "de431.eph")) or \
       os.path.exists(os.path.join(ephe_path, "de406.eph")) or \
       os.path.exists(os.path.join(ephe_path, "de200.eph")):
        print("JPL ephemeris file found")
    else:
        print("No JPL ephemeris files found. Using compressed files only.")
        print("For highest precision, consider downloading JPL files.")
    
    # Check for asteroid directories
    for i in range(10):  # Check first 10 asteroid directories (ast0-ast9)
        ast_dir = os.path.join(ephe_path, f"ast{i}")
        if os.path.isdir(ast_dir):
            print(f"Found asteroid directory: ast{i}")
            # Check if any asteroid files exist
            files = [f for f in os.listdir(ast_dir) if f.endswith(".se1")]
            if files:
                print(f"  Contains {len(files)} asteroid files")
            else:
                print(f"  Directory exists but contains no asteroid files")
        else:
            print(f"Missing asteroid directory: ast{i}")
    
    print("Ephemeris path initialization completed")


def julian_day_ut(year: int, month: int, day: int, hour: float = 0.0) -> float:
    """Convert a date and time to Julian Day (UT)"""
    return swe.julday_ut(year, month, day, hour)


def julian_day(year: int, month: int, day: int, hour: float = 0.0) -> float:
    """Convert a date and time to Julian Day (ET)"""
    return swe.julday(year, month, day, hour)


def parse_datetime(date_str: str, time_str: Optional[str] = None) -> Tuple[int, int, int, float]:
    """
    Parse date and time strings into components for Julian day calculation
    
    Args:
        date_str: Date in YYYY-MM-DD format
        time_str: Time in HH:MM:SS format, optional (defaults to 00:00:00)
    
    Returns:
        Tuple of (year, month, day, hour_decimal)
    """
    if not time_str:
        time_str = "00:00:00"
    
    # Parse date
    year, month, day = map(int, date_str.split("-"))
    
    # Parse time
    hour, minute, second = map(int, time_str.split(":"))
    hour_decimal = hour + minute/60 + second/3600
    
    return year, month, day, hour_decimal


def calculate_planet_position(jd: float, planet_id: int) -> Dict[str, float]:
    """
    Calculate planet position for a given Julian day
    
    Args:
        jd: Julian day
        planet_id: Planet ID as per Swiss Ephemeris
    
    Returns:
        Dictionary with planet position data
    """
    xx, ret = swe.calc_ut(jd, planet_id)
    return {
        "longitude": xx[0],
        "latitude": xx[1],
        "distance": xx[2],
        "longitude_speed": xx[3],
        "latitude_speed": xx[4],
        "distance_speed": xx[5]
    }


def get_planet_id(planet: PlanetType) -> int:
    """
    Convert PlanetType enum to Swiss Ephemeris planet ID
    
    Args:
        planet: PlanetType enum value
    
    Returns:
        Swiss Ephemeris planet ID
    """
    planet_map = {
        PlanetType.SUN: swe.SUN,
        PlanetType.MOON: swe.MOON,
        PlanetType.MERCURY: swe.MERCURY,
        PlanetType.VENUS: swe.VENUS,
        PlanetType.MARS: swe.MARS,
        PlanetType.JUPITER: swe.JUPITER,
        PlanetType.SATURN: swe.SATURN,
        PlanetType.URANUS: swe.URANUS,
        PlanetType.NEPTUNE: swe.NEPTUNE,
        PlanetType.PLUTO: swe.PLUTO,
        PlanetType.MEAN_NODE: swe.MEAN_NODE,
        PlanetType.TRUE_NODE: swe.TRUE_NODE,
        PlanetType.MEAN_APOG: swe.MEAN_APOG,
        PlanetType.OSCU_APOG: swe.OSCU_APOG,
        PlanetType.EARTH: swe.EARTH,
        PlanetType.CHIRON: swe.CHIRON,
        PlanetType.PHOLUS: swe.PHOLUS,
        PlanetType.CERES: swe.CERES,
        PlanetType.PALLAS: swe.PALLAS,
        PlanetType.JUNO: swe.JUNO,
        PlanetType.VESTA: swe.VESTA
    }
    return planet_map[planet]


def calculate_houses(jd: float, lat: float, lon: float, hsys: str = 'P') -> Dict[str, Any]:
    """
    Calculate house cusps for a given Julian day and location
    
    Args:
        jd: Julian day
        lat: Latitude in degrees (positive for North, negative for South)
        lon: Longitude in degrees (positive for East, negative for West)
        hsys: House system character code
    
    Returns:
        Dictionary with house cusps and special points
    """
    cusps, ascmc = swe.houses(jd, lat, lon, hsys.encode('utf-8'))
    
    # Get additional points
    eps = swe.calc_ut(jd, swe.ECL_NUT)[0]
    
    # Calculate additional points
    equatorial_ascendant = swe.house_pos(ascmc[0], lat, eps[0], 'E')
    co_asc_koch = swe.house_pos(ascmc[0] + 180, lat, eps[0], 'K')
    co_asc_munkasey = swe.house_pos(ascmc[0] + 180, -lat, eps[0], 'K')
    polar_asc = swe.house_pos(ascmc[0], -lat, eps[0], 'P')
    
    return {
        "house_cusps": list(cusps),
        "ascendant": ascmc[0],
        "mc": ascmc[1],
        "armc": ascmc[2],
        "vertex": ascmc[3],
        "equatorial_ascendant": equatorial_ascendant,
        "co_ascendant_koch": co_asc_koch,
        "co_ascendant_munkasey": co_asc_munkasey,
        "polar_ascendant": polar_asc
    }


def get_aspect_angle(aspect_type: AspectType) -> float:
    """
    Get the angle in degrees for a given aspect type
    
    Args:
        aspect_type: Type of aspect
    
    Returns:
        Angle in degrees
    """
    aspect_angles = {
        AspectType.CONJUNCTION: 0.0,
        AspectType.SEXTILE: 60.0,
        AspectType.SQUARE: 90.0,
        AspectType.TRINE: 120.0,
        AspectType.OPPOSITION: 180.0,
        AspectType.QUINCUNX: 150.0,
        AspectType.SEMISEXTILE: 30.0,
        AspectType.SEMISQUARE: 45.0,
        AspectType.SESQUIQUADRATE: 135.0,
        AspectType.QUINTILE: 72.0,
        AspectType.BIQUINTILE: 144.0
    }
    return aspect_angles[aspect_type]


def get_aspect_type(angle: float, orb_limit: float = 8.0) -> Optional[AspectType]:
    """
    Determine the aspect type based on the angle between two planets
    
    Args:
        angle: Angle between two planets in degrees
        orb_limit: Maximum orb to consider, in degrees
    
    Returns:
        AspectType or None if no aspect is found
    """
    angles = {
        0.0: AspectType.CONJUNCTION,
        60.0: AspectType.SEXTILE,
        90.0: AspectType.SQUARE,
        120.0: AspectType.TRINE,
        180.0: AspectType.OPPOSITION,
        150.0: AspectType.QUINCUNX,
        30.0: AspectType.SEMISEXTILE,
        45.0: AspectType.SEMISQUARE,
        135.0: AspectType.SESQUIQUADRATE,
        72.0: AspectType.QUINTILE,
        144.0: AspectType.BIQUINTILE
    }
    
    for base_angle, aspect in angles.items():
        orb = abs((angle % 360) - base_angle)
        if orb > 180:
            orb = 360 - orb
        if orb <= orb_limit:
            return aspect
    
    return None


def calculate_orb(angle1: float, angle2: float) -> float:
    """
    Calculate the orb (difference) between two angles
    
    Args:
        angle1: First angle in degrees
        angle2: Second angle in degrees
    
    Returns:
        Orb in degrees
    """
    diff = abs(angle1 - angle2)
    if diff > 180:
        diff = 360 - diff
    return diff


def calculate_aspects(planet_positions: Dict[PlanetType, Dict[str, float]], 
                      orb_limit: float = 8.0) -> List[Aspect]:
    """
    Calculate aspects between planets
    
    Args:
        planet_positions: Dictionary of planet positions
        orb_limit: Maximum orb to consider, in degrees
    
    Returns:
        List of aspects
    """
    aspects = []
    
    # Get all planet combinations
    planets = list(planet_positions.keys())
    for i, p1 in enumerate(planets):
        for p2 in planets[i+1:]:  # Avoid duplicates and self-aspects
            pos1 = planet_positions[p1]["longitude"]
            pos2 = planet_positions[p2]["longitude"]
            speed1 = planet_positions[p1]["longitude_speed"]
            speed2 = planet_positions[p2]["longitude_speed"]
            
            # Calculate angular difference
            angle = (pos2 - pos1) % 360
            
            # Determine aspect type
            aspect_type = get_aspect_type(angle, orb_limit)
            if aspect_type:
                # Calculate exact aspect angle
                exact_angle = get_aspect_angle(aspect_type)
                
                # Calculate orb
                orb = calculate_orb(angle, exact_angle)
                
                # Determine if applying or separating
                relative_speed = speed2 - speed1
                is_applying = False
                
                if exact_angle == 0:  # Conjunction
                    is_applying = angle < 180 and relative_speed < 0 or angle >= 180 and relative_speed > 0
                elif exact_angle == 180:  # Opposition
                    is_applying = angle < 180 and relative_speed > 0 or angle >= 180 and relative_speed < 0
                else:
                    # For other aspects, the logic is more complex and depends on the houses
                    # This is a simplified version
                    if angle < exact_angle:
                        is_applying = relative_speed > 0
                    else:
                        is_applying = relative_speed < 0
                
                aspects.append(Aspect(
                    planet1=p1,
                    planet2=p2,
                    aspect_type=aspect_type,
                    angle=angle,
                    orb=orb,
                    is_applying=is_applying
                ))
    
    return aspects


def is_retrograde(speed: float) -> bool:
    """
    Check if a planet is retrograde based on its speed
    
    Args:
        speed: Longitude speed of the planet
    
    Returns:
        True if retrograde, False otherwise
    """
    return speed < 0
