#!/usr/bin/env python3
"""
Swiss Ephemeris API Server - All-In-One Deployment

This script contains a complete FastAPI application for astrological calculations
along with embedded Swiss Ephemeris files. It extracts the ephemeris files to a 
temporary directory at runtime and starts the API server.

Usage:
    python astro_api_server.py [--port PORT] [--host HOST] [--api-key API_KEY]

Options:
    --port PORT     Port to run the server on (default: 8000)
    --host HOST     Host to bind the server to (default: 0.0.0.0)
    --api-key KEY   API key for authentication (optional, generate random if not provided)
"""

import argparse
import asyncio
import base64
import datetime
import enum
import io
import json
import logging
import os
import platform
import shutil
import sys
import tempfile
import time
import zipfile
from enum import Enum
from typing import Any, Dict, List, Optional, Tuple, Union

# Check for required packages and install if missing
required_packages = [
    "fastapi", "uvicorn", "pyswisseph", "pydantic"
]

missing_packages = []
for package in required_packages:
    try:
        __import__(package)
    except ImportError:
        missing_packages.append(package)

if missing_packages:
    print(f"Installing required packages: {', '.join(missing_packages)}")
    import subprocess
    subprocess.check_call([
        sys.executable, "-m", "pip", "install", "--quiet", *missing_packages
    ])

# Now import the required packages
import uvicorn
from fastapi import Depends, FastAPI, HTTPException, Query, Request
from fastapi.middleware.cors import CORSMiddleware
from fastapi.responses import JSONResponse
from fastapi.security.api_key import APIKey, APIKeyHeader
from pydantic import BaseModel, Field
import swisseph as swe

# Configure logging
logging.basicConfig(
    level=logging.INFO,
    format='%(asctime)s - %(name)s - %(levelname)s - %(message)s'
)
logger = logging.getLogger("astro_api")

# Helper function to generate a random API key
def generate_api_key(length=32):
    import random
    import string
    chars = string.ascii_letters + string.digits
    return ''.join(random.choice(chars) for _ in range(length))

# Parse command line arguments
parser = argparse.ArgumentParser(description="Swiss Ephemeris API Server")
parser.add_argument("--port", type=int, default=8000, help="Port to run the server on")
parser.add_argument("--host", type=str, default="0.0.0.0", help="Host to bind the server to")
parser.add_argument("--api-key", type=str, help="API key for authentication (optional)")
args = parser.parse_args()

# Set API key for authentication
API_KEY = args.api_key or generate_api_key()
if not args.api_key:
    print(f"\nGenerated API key: {API_KEY}\n")

# API Key security scheme
api_key_header = APIKeyHeader(name="X-API-Key", auto_error=False)

async def get_api_key(api_key_header: str = Depends(api_key_header)):
    if not API_KEY:
        return None
    if api_key_header == API_KEY:
        return api_key_header
    raise HTTPException(
        status_code=401,
        detail="Invalid API Key",
        headers={"WWW-Authenticate": "API key required in X-API-Key header"},
    )

#--------------------------------------------------------------------------#
# Ephemeris File Extraction                                                #
#--------------------------------------------------------------------------#

# Import the ephemeris data from the generated file
try:
    from ephemeris_data import EPHEMERIS_FILES
except ImportError:
    # Fallback if file doesn't exist
    logger.warning("ephemeris_data.py not found. Using empty EPHEMERIS_FILES.")
    EPHEMERIS_FILES = {}

def extract_ephemeris_files():
    """Extract embedded ephemeris files to a temporary directory"""
    # Create temporary directory for ephemeris files
    temp_dir = tempfile.mkdtemp(prefix="swisseph_")
    logger.info(f"Created temporary directory for ephemeris files: {temp_dir}")
    
    # Extract files to temporary directory
    for filename, content in EPHEMERIS_FILES.items():
        # Skip empty content or placeholders
        if not content.strip() or "# BASE64 ENCODED CONTENT WILL BE INSERTED HERE" in content:
            logger.warning(f"Skipping empty or placeholder content for {filename}")
            continue
        
        try:
            file_path = os.path.join(temp_dir, filename)
            with open(file_path, "wb") as f:
                f.write(base64.b64decode(content.strip()))
            logger.info(f"Extracted {filename} to {file_path}")
        except Exception as e:
            logger.error(f"Failed to extract {filename}: {str(e)}")
    
    # Create asteroid directories
    for i in range(3):
        os.makedirs(os.path.join(temp_dir, f"ast{i}"), exist_ok=True)
    
    return temp_dir

#--------------------------------------------------------------------------#
# FastAPI Models                                                           #
#--------------------------------------------------------------------------#

class PlanetType(str, Enum):
    """Enumeration of planet types for clearer API interface"""
    SUN = "sun"  # SE_SUN = 0
    MOON = "moon"  # SE_MOON = 1
    MERCURY = "mercury"  # SE_MERCURY = 2
    VENUS = "venus"  # SE_VENUS = 3
    MARS = "mars"  # SE_MARS = 4
    JUPITER = "jupiter"  # SE_JUPITER = 5
    SATURN = "saturn"  # SE_SATURN = 6
    URANUS = "uranus"  # SE_URANUS = 7
    NEPTUNE = "neptune"  # SE_NEPTUNE = 8
    PLUTO = "pluto"  # SE_PLUTO = 9
    MEAN_NODE = "mean_node"  # SE_MEAN_NODE = 10
    TRUE_NODE = "true_node"  # SE_TRUE_NODE = 11
    CHIRON = "chiron"  # SE_CHIRON = 15
    CERES = "ceres"  # SE_CERES = 17
    PALLAS = "pallas"  # SE_PALLAS = 18
    JUNO = "juno"  # SE_JUNO = 19
    VESTA = "vesta"  # SE_VESTA = 20

class HouseSystem(str, Enum):
    """Enumeration of house systems"""
    PLACIDUS = "P"
    KOCH = "K"
    PORPHYRIUS = "O"
    REGIOMONTANUS = "R"
    CAMPANUS = "C"
    EQUAL = "E"
    WHOLE_SIGN = "W"
    MERIDIAN = "X"
    MORINUS = "M"
    KRUSINSKI = "U"
    ALCABITIUS = "H"
    POLICH_PAGE = "T"

class AspectType(str, Enum):
    """Enumeration of aspect types"""
    CONJUNCTION = "conjunction"
    SEXTILE = "sextile"
    SQUARE = "square"
    TRINE = "trine"
    OPPOSITION = "opposition"
    QUINCUNX = "quincunx"
    SEMISEXTILE = "semisextile"
    SEMISQUARE = "semisquare"
    SESQUIQUADRATE = "sesquiquadrate"
    QUINTILE = "quintile"
    BIQUINTILE = "biquintile"

class PlanetPosition(BaseModel):
    """Model for planet positions"""
    longitude: float = Field(..., description="Longitude in degrees")
    latitude: float = Field(..., description="Latitude in degrees")
    distance: float = Field(..., description="Distance in AU")
    longitude_speed: float = Field(..., description="Daily speed in longitude (degrees/day)")
    latitude_speed: float = Field(..., description="Daily speed in latitude (degrees/day)")
    distance_speed: float = Field(..., description="Daily speed in distance (AU/day)")
    is_retrograde: bool = Field(..., description="Whether the planet is retrograde")
    sign: str = Field(..., description="Zodiac sign")
    sign_longitude: float = Field(..., description="Longitude within the sign (0-30 degrees)")
    
class HouseCusps(BaseModel):
    """Model for house cusps"""
    house_cusps: List[float] = Field(..., description="House cusps in degrees")
    ascendant: float = Field(..., description="Ascendant in degrees")
    mc: float = Field(..., description="Medium Coeli (MC) in degrees")
    armc: float = Field(..., description="ARMC in degrees")
    vertex: float = Field(..., description="Vertex in degrees")
    equatorial_ascendant: Optional[float] = Field(None, description="Equatorial Ascendant in degrees")
    co_ascendant_koch: Optional[float] = Field(None, description="Co-Ascendant (Koch) in degrees")
    co_ascendant_munkasey: Optional[float] = Field(None, description="Co-Ascendant (Munkasey) in degrees")
    polar_ascendant: Optional[float] = Field(None, description="Polar Ascendant in degrees")

class Aspect(BaseModel):
    """Model for aspects between planets"""
    planet1: PlanetType
    planet2: PlanetType
    aspect_type: AspectType
    angle: float = Field(..., description="Exact angle of the aspect in degrees")
    orb: float = Field(..., description="Orb (deviation from exact aspect) in degrees")
    is_applying: bool = Field(..., description="True if the aspect is applying, False if separating")
    is_exact: bool = Field(..., description="True if the orb is less than 1 degree")

class TransitInfo(BaseModel):
    """Model for transit information"""
    transit_date: str
    planet: PlanetType
    natal_position: float = Field(..., description="Natal position in degrees")
    transit_position: float = Field(..., description="Transit position in degrees")
    aspect_angle: Optional[float] = Field(None, description="Aspect angle if any")
    aspect_type: Optional[AspectType] = Field(None, description="Type of aspect if any")
    orb: Optional[float] = Field(None, description="Orb of aspect if any")
    is_retrograde: bool = Field(..., description="Whether the transiting planet is retrograde")

class NatalChart(BaseModel):
    """Model for natal chart data"""
    planets: Dict[str, PlanetPosition]
    houses: HouseCusps
    aspects: List[Aspect] = []
    date: str = Field(..., description="Birth date in YYYY-MM-DD format")
    time: str = Field(..., description="Birth time in HH:MM:SS format")
    latitude: float = Field(..., description="Birth latitude in degrees")
    longitude: float = Field(..., description="Birth longitude in degrees")

#--------------------------------------------------------------------------#
# Swiss Ephemeris Utility Functions                                        #
#--------------------------------------------------------------------------#

def set_ephemeris_path(path: Optional[str] = None) -> None:
    """Set the path to the Swiss Ephemeris files"""
    if path:
        ephe_path = path
    else:
        # Use the path from environment variable or default
        ephe_path = os.environ.get("EPHE_PATH", tempfile.gettempdir())
    
    swe.set_ephe_path(ephe_path)
    logger.info(f"Ephemeris path set to: {ephe_path}")
    
    # Verify that the main ephemeris files can be found
    try:
        jd = swe.julday(2000, 1, 1, 12)
        pos, flags = swe.calc_ut(jd, swe.SUN)
        logger.info("Successfully found ephemeris files")
    except Exception as e:
        logger.error(f"Error: Failed to calculate position - {str(e)}")
        logger.error("This might indicate that the ephemeris files cannot be found")
    
    logger.info("Ephemeris path initialization completed")

def julian_day_ut(year: int, month: int, day: int, hour: float = 0.0) -> float:
    """Convert a date and time to Julian Day (UT)"""
    return swe.julday_ut(year, month, day, hour)

def parse_datetime(date_str: str, time_str: Optional[str] = None) -> Tuple[int, int, int, float]:
    """
    Parse date and time strings into components for Julian day calculation
    
    Args:
        date_str: Date in YYYY-MM-DD format
        time_str: Time in HH:MM:SS format, optional (defaults to 00:00:00)
    
    Returns:
        Tuple of (year, month, day, hour_decimal)
    """
    if not time_str:
        time_str = "00:00:00"
    
    # Parse date
    year, month, day = map(int, date_str.split("-"))
    
    # Parse time
    hour, minute, second = map(int, time_str.split(":"))
    hour_decimal = hour + minute/60 + second/3600
    
    return year, month, day, hour_decimal

def get_planet_id(planet: PlanetType) -> int:
    """
    Convert PlanetType enum to Swiss Ephemeris planet ID
    """
    planet_map = {
        PlanetType.SUN: swe.SUN,
        PlanetType.MOON: swe.MOON,
        PlanetType.MERCURY: swe.MERCURY,
        PlanetType.VENUS: swe.VENUS,
        PlanetType.MARS: swe.MARS,
        PlanetType.JUPITER: swe.JUPITER,
        PlanetType.SATURN: swe.SATURN,
        PlanetType.URANUS: swe.URANUS,
        PlanetType.NEPTUNE: swe.NEPTUNE,
        PlanetType.PLUTO: swe.PLUTO,
        PlanetType.MEAN_NODE: swe.MEAN_NODE,
        PlanetType.TRUE_NODE: swe.TRUE_NODE,
        PlanetType.CHIRON: swe.CHIRON,
        PlanetType.CERES: swe.CERES,
        PlanetType.PALLAS: swe.PALLAS,
        PlanetType.JUNO: swe.JUNO,
        PlanetType.VESTA: swe.VESTA
    }
    return planet_map[planet]

def get_zodiac_sign(longitude: float) -> Tuple[str, float]:
    """
    Get the zodiac sign and position within the sign for a given longitude
    
    Args:
        longitude: Longitude in degrees
    
    Returns:
        Tuple of (sign_name, sign_position)
    """
    signs = [
        "Aries", "Taurus", "Gemini", "Cancer", 
        "Leo", "Virgo", "Libra", "Scorpio", 
        "Sagittarius", "Capricorn", "Aquarius", "Pisces"
    ]
    
    sign_num = int(longitude / 30) % 12
    sign_pos = longitude % 30
    
    return signs[sign_num], sign_pos

def calculate_planet_position(jd: float, planet_id: int) -> Dict[str, Any]:
    """
    Calculate planet position for a given Julian day
    
    Args:
        jd: Julian day
        planet_id: Planet ID as per Swiss Ephemeris
    
    Returns:
        Dictionary with planet position data
    """
    xx, ret = swe.calc_ut(jd, planet_id)
    
    # Get zodiac sign
    sign, sign_pos = get_zodiac_sign(xx[0])
    
    return {
        "longitude": xx[0],
        "latitude": xx[1],
        "distance": xx[2],
        "longitude_speed": xx[3],
        "latitude_speed": xx[4],
        "distance_speed": xx[5],
        "is_retrograde": xx[3] < 0,  # Retrograde if longitude speed is negative
        "sign": sign,
        "sign_longitude": sign_pos
    }

def calculate_houses(jd: float, lat: float, lon: float, hsys: str = 'P') -> Dict[str, Any]:
    """
    Calculate house cusps for a given Julian day and location
    
    Args:
        jd: Julian day
        lat: Latitude in degrees (positive for North, negative for South)
        lon: Longitude in degrees (positive for East, negative for West)
        hsys: House system character code
    
    Returns:
        Dictionary with house cusps and special points
    """
    try:
        cusps, ascmc = swe.houses(jd, lat, lon, hsys.encode('utf-8'))
        
        # Get additional points if available
        result = {
            "house_cusps": list(cusps),
            "ascendant": ascmc[0],
            "mc": ascmc[1],
            "armc": ascmc[2],
            "vertex": ascmc[3],
        }
        
        # Calculate additional points if possible
        try:
            eps = swe.calc_ut(jd, swe.ECL_NUT)[0]
            
            equatorial_ascendant = swe.house_pos(ascmc[0], lat, eps[0], 'E')
            co_asc_koch = swe.house_pos(ascmc[0] + 180, lat, eps[0], 'K')
            co_asc_munkasey = swe.house_pos(ascmc[0] + 180, -lat, eps[0], 'K')
            polar_asc = swe.house_pos(ascmc[0], -lat, eps[0], 'P')
            
            result.update({
                "equatorial_ascendant": equatorial_ascendant,
                "co_ascendant_koch": co_asc_koch,
                "co_ascendant_munkasey": co_asc_munkasey,
                "polar_ascendant": polar_asc
            })
        except Exception as e:
            logger.warning(f"Could not calculate additional house points: {str(e)}")
        
        return result
    except Exception as e:
        logger.error(f"Error calculating houses: {str(e)}")
        raise ValueError(f"Could not calculate houses: {str(e)}")

def get_aspect_angle(aspect_type: AspectType) -> float:
    """Get the angle in degrees for a given aspect type"""
    aspect_angles = {
        AspectType.CONJUNCTION: 0.0,
        AspectType.SEXTILE: 60.0,
        AspectType.SQUARE: 90.0,
        AspectType.TRINE: 120.0,
        AspectType.OPPOSITION: 180.0,
        AspectType.QUINCUNX: 150.0,
        AspectType.SEMISEXTILE: 30.0,
        AspectType.SEMISQUARE: 45.0,
        AspectType.SESQUIQUADRATE: 135.0,
        AspectType.QUINTILE: 72.0,
        AspectType.BIQUINTILE: 144.0
    }
    return aspect_angles[aspect_type]

def get_aspect_type(angle: float, orb_limit: float = 8.0) -> Optional[AspectType]:
    """
    Determine the aspect type based on the angle between two planets
    
    Args:
        angle: Angle between two planets in degrees
        orb_limit: Maximum orb to consider, in degrees
    
    Returns:
        AspectType or None if no aspect is found
    """
    angles = {
        0.0: AspectType.CONJUNCTION,
        60.0: AspectType.SEXTILE,
        90.0: AspectType.SQUARE,
        120.0: AspectType.TRINE,
        180.0: AspectType.OPPOSITION,
        150.0: AspectType.QUINCUNX,
        30.0: AspectType.SEMISEXTILE,
        45.0: AspectType.SEMISQUARE,
        135.0: AspectType.SESQUIQUADRATE,
        72.0: AspectType.QUINTILE,
        144.0: AspectType.BIQUINTILE
    }
    
    for base_angle, aspect in angles.items():
        orb = abs((angle % 360) - base_angle)
        if orb > 180:
            orb = 360 - orb
        if orb <= orb_limit:
            return aspect
    
    return None

def calculate_orb(angle1: float, angle2: float) -> float:
    """Calculate the orb (difference) between two angles"""
    diff = abs(angle1 - angle2)
    if diff > 180:
        diff = 360 - diff
    return diff

def calculate_aspects(planet_positions: Dict[PlanetType, Dict[str, Any]], 
                     orb_limit: float = 8.0) -> List[Aspect]:
    """
    Calculate aspects between planets
    
    Args:
        planet_positions: Dictionary of planet positions
        orb_limit: Maximum orb to consider, in degrees
    
    Returns:
        List of aspects
    """
    aspects = []
    
    # Get all planet combinations
    planets = list(planet_positions.keys())
    for i, p1 in enumerate(planets):
        for p2 in planets[i+1:]:  # Avoid duplicates and self-aspects
            pos1 = planet_positions[p1]["longitude"]
            pos2 = planet_positions[p2]["longitude"]
            speed1 = planet_positions[p1]["longitude_speed"]
            speed2 = planet_positions[p2]["longitude_speed"]
            
            # Calculate angular difference
            angle = (pos2 - pos1) % 360
            
            # Determine aspect type
            aspect_type = get_aspect_type(angle, orb_limit)
            if aspect_type:
                # Calculate exact aspect angle
                exact_angle = get_aspect_angle(aspect_type)
                
                # Calculate orb
                orb = calculate_orb(angle, exact_angle)
                
                # Determine if applying or separating
                relative_speed = speed2 - speed1
                is_applying = False
                
                if exact_angle == 0:  # Conjunction
                    is_applying = angle < 180 and relative_speed < 0 or angle >= 180 and relative_speed > 0
                elif exact_angle == 180:  # Opposition
                    is_applying = angle < 180 and relative_speed > 0 or angle >= 180 and relative_speed < 0
                else:
                    # For other aspects, the logic is more complex and depends on the houses
                    # This is a simplified version
                    if angle < exact_angle:
                        is_applying = relative_speed > 0
                    else:
                        is_applying = relative_speed < 0
                
                aspects.append(Aspect(
                    planet1=p1,
                    planet2=p2,
                    aspect_type=aspect_type,
                    angle=angle,
                    orb=orb,
                    is_applying=is_applying,
                    is_exact=orb < 1.0
                ))
    
    return aspects

#--------------------------------------------------------------------------#
# FastAPI Application                                                      #
#--------------------------------------------------------------------------#

# Initialize FastAPI app
app = FastAPI(
    title="Swiss Ephemeris API",
    description="API for astrological calculations using Swiss Ephemeris",
    version="1.0.0",
    docs_url="/docs",
    redoc_url="/redoc",
)

# Add CORS middleware
app.add_middleware(
    CORSMiddleware,
    allow_origins=["*"],  # Change this in production to specific origins
    allow_credentials=True,
    allow_methods=["*"],
    allow_headers=["*"],
)

# Set ephemeris path at startup
@app.on_event("startup")
async def startup_event():
    # Extract ephemeris files to temporary directory
    temp_dir = extract_ephemeris_files()
    
    # Set environment variable for other parts of the app
    os.environ["EPHE_PATH"] = temp_dir
    
    # Set the ephemeris path
    set_ephemeris_path(temp_dir)
    logger.info(f"Swiss Ephemeris path set to: {temp_dir}")

# Clean up temporary directory on shutdown
@app.on_event("shutdown")
async def shutdown_event():
    temp_dir = os.environ.get("EPHE_PATH")
    if temp_dir and os.path.exists(temp_dir) and os.path.isdir(temp_dir):
        shutil.rmtree(temp_dir)
        logger.info(f"Removed temporary directory: {temp_dir}")

@app.get("/")
async def root():
    """
    Root endpoint that returns a welcome message and basic API info
    """
    return {
        "message": "Welcome to the Swiss Ephemeris API", 
        "docs": "/docs",
        "version": "1.0.0",
        "timezone": "All calculations use UTC"
    }

@app.get("/planet/{planet}", response_model=PlanetPosition, tags=["Planets"])
async def get_planet_position(
    planet: PlanetType,
    date: str = Query(..., description="Date in YYYY-MM-DD format"),
    time: Optional[str] = Query("00:00:00", description="Time in HH:MM:SS format"),
    geo_lat: Optional[float] = Query(0.0, description="Geographical latitude (-90 to +90)"),
    geo_lon: Optional[float] = Query(0.0, description="Geographical longitude (-180 to +180)"),
    apparent: Optional[bool] = Query(True, description="Whether to calculate apparent position"),
    api_key: APIKey = Depends(get_api_key),
):
    """
    Get the position of a planet at a specific date and time.
    
    - **planet**: Planet name (sun, moon, mercury, etc.)
    - **date**: Date in YYYY-MM-DD format
    - **time**: Time in HH:MM:SS format (default: 00:00:00)
    - **geo_lat**: Geographical latitude (-90 to +90, default: 0.0)
    - **geo_lon**: Geographical longitude (-180 to +180, default: 0.0)
    - **apparent**: Whether to calculate apparent position (true) or true position (false)
    """
    try:
        # Parse date and time
        year, month, day, hour = parse_datetime(date, time)
        
        # Convert to Julian day
        jd = julian_day_ut(year, month, day, hour)
        
        # Set calculation flags
        flags = swe.FLG_SWIEPH
        if apparent:
            flags |= swe.FLG_TRUEPOS
        else:
            flags |= swe.FLG_SWIEPH
        
        # Calculate planet position
        planet_id = get_planet_id(planet)
        xx, ret = swe.calc_ut(jd, planet_id, flags)
        
        # Get zodiac sign
        sign, sign_pos = get_zodiac_sign(xx[0])
        
        # Return planet position
        return PlanetPosition(
            longitude=xx[0],
            latitude=xx[1],
            distance=xx[2],
            longitude_speed=xx[3],
            latitude_speed=xx[4],
            distance_speed=xx[5],
            is_retrograde=xx[3] < 0,
            sign=sign,
            sign_longitude=sign_pos
        )
    except Exception as e:
        logger.error(f"Error calculating planet position: {str(e)}")
        raise HTTPException(status_code=400, detail=str(e))

@app.get("/planets", response_model=Dict[str, PlanetPosition], tags=["Planets"])
async def get_all_planets(
    date: str = Query(..., description="Date in YYYY-MM-DD format"),
    time: Optional[str] = Query("00:00:00", description="Time in HH:MM:SS format"),
    geo_lat: Optional[float] = Query(0.0, description="Geographical latitude (-90 to +90)"),
    geo_lon: Optional[float] = Query(0.0, description="Geographical longitude (-180 to +180)"),
    apparent: Optional[bool] = Query(True, description="Whether to calculate apparent position"),
    api_key: APIKey = Depends(get_api_key),
):
    """
    Get the positions of all major planets at a specific date and time.
    
    - **date**: Date in YYYY-MM-DD format
    - **time**: Time in HH:MM:SS format (default: 00:00:00)
    - **geo_lat**: Geographical latitude (-90 to +90, default: 0.0)
    - **geo_lon**: Geographical longitude (-180 to +180, default: 0.0)
    - **apparent**: Whether to calculate apparent position (true) or true position (false)
    """
    try:
        # Parse date and time
        year, month, day, hour = parse_datetime(date, time)
        
        # Convert to Julian day
        jd = julian_day_ut(year, month, day, hour)
        
        # Set calculation flags
        flags = swe.FLG_SWIEPH
        if apparent:
            flags |= swe.FLG_TRUEPOS
        else:
            flags |= swe.FLG_SWIEPH
        
        # Get all planets
        result = {}
        for planet in PlanetType:
            planet_id = get_planet_id(planet)
            try:
                xx, ret = swe.calc_ut(jd, planet_id, flags)
                
                # Get zodiac sign
                sign, sign_pos = get_zodiac_sign(xx[0])
                
                result[planet.value] = PlanetPosition(
                    longitude=xx[0],
                    latitude=xx[1],
                    distance=xx[2],
                    longitude_speed=xx[3],
                    latitude_speed=xx[4],
                    distance_speed=xx[5],
                    is_retrograde=xx[3] < 0,
                    sign=sign,
                    sign_longitude=sign_pos
                )
            except Exception as e:
                logger.warning(f"Could not calculate position for planet {planet.value}: {str(e)}")
        
        return result
    except Exception as e:
        logger.error(f"Error calculating planet positions: {str(e)}")
        raise HTTPException(status_code=400, detail=str(e))

@app.get("/houses", response_model=HouseCusps, tags=["Houses"])
async def get_houses(
    date: str = Query(..., description="Date in YYYY-MM-DD format"),
    time: Optional[str] = Query("00:00:00", description="Time in HH:MM:SS format"),
    geo_lat: float = Query(..., description="Geographical latitude (-90 to +90)"),
    geo_lon: float = Query(..., description="Geographical longitude (-180 to +180)"),
    house_system: HouseSystem = Query(HouseSystem.PLACIDUS, description="House system to use"),
    api_key: APIKey = Depends(get_api_key),
):
    """
    Calculate house cusps for a specific date, time, and location.
    
    - **date**: Date in YYYY-MM-DD format
    - **time**: Time in HH:MM:SS format (default: 00:00:00)
    - **geo_lat**: Geographical latitude (-90 to +90)
    - **geo_lon**: Geographical longitude (-180 to +180)
    - **house_system**: House system to use (default: Placidus)
    """
    try:
        # Parse date and time
        year, month, day, hour = parse_datetime(date, time)
        
        # Convert to Julian day
        jd = julian_day_ut(year, month, day, hour)
        
        # Calculate houses
        houses = calculate_houses(jd, geo_lat, geo_lon, house_system.value)
        
        return HouseCusps(**houses)
    except Exception as e:
        logger.error(f"Error calculating houses: {str(e)}")
        raise HTTPException(status_code=400, detail=str(e))

@app.get("/aspects", response_model=List[Aspect], tags=["Aspects"])
async def get_aspects(
    date: str = Query(..., description="Date in YYYY-MM-DD format"),
    time: Optional[str] = Query("00:00:00", description="Time in HH:MM:SS format"),
    orb: Optional[float] = Query(8.0, description="Maximum orb in degrees"),
    include_planets: List[PlanetType] = Query(
        [
            PlanetType.SUN, PlanetType.MOON, PlanetType.MERCURY, 
            PlanetType.VENUS, PlanetType.MARS, PlanetType.JUPITER, 
            PlanetType.SATURN, PlanetType.URANUS, PlanetType.NEPTUNE, 
            PlanetType.PLUTO
        ],
        description="Planets to include in aspect calculations"
    ),
    api_key: APIKey = Depends(get_api_key),
):
    """
    Calculate aspects between planets for a specific date and time.
    
    - **date**: Date in YYYY-MM-DD format
    - **time**: Time in HH:MM:SS format (default: 00:00:00)
    - **orb**: Maximum orb in degrees (default: 8.0)
    - **include_planets**: List of planets to include in aspect calculations
    """
    try:
        # Parse date and time
        year, month, day, hour = parse_datetime(date, time)
        
        # Convert to Julian day
        jd = julian_day_ut(year, month, day, hour)
        
        # Calculate planet positions
        positions = {}
        for planet in include_planets:
            planet_id = get_planet_id(planet)
            try:
                position = calculate_planet_position(jd, planet_id)
                positions[planet] = position
            except Exception as e:
                logger.warning(f"Could not calculate position for planet {planet.value}: {str(e)}")
        
        # Calculate aspects
        aspects = calculate_aspects(positions, orb)
        
        return aspects
    except Exception as e:
        logger.error(f"Error calculating aspects: {str(e)}")
        raise HTTPException(status_code=400, detail=str(e))

@app.get("/chart", response_model=NatalChart, tags=["Charts"])
async def get_natal_chart(
    date: str = Query(..., description="Date in YYYY-MM-DD format"),
    time: Optional[str] = Query("00:00:00", description="Time in HH:MM:SS format"),
    geo_lat: float = Query(..., description="Geographical latitude (-90 to +90)"),
    geo_lon: float = Query(..., description="Geographical longitude (-180 to +180)"),
    house_system: HouseSystem = Query(HouseSystem.PLACIDUS, description="House system to use"),
    orb: Optional[float] = Query(8.0, description="Maximum orb for aspects in degrees"),
    include_planets: List[PlanetType] = Query(
        None,  # Default to all planets if not specified
        description="Planets to include in chart"
    ),
    api_key: APIKey = Depends(get_api_key),
):
    """
    Calculate a complete natal chart for a specific date, time, and location.
    
    - **date**: Date in YYYY-MM-DD format
    - **time**: Time in HH:MM:SS format (default: 00:00:00)
    - **geo_lat**: Geographical latitude (-90 to +90)
    - **geo_lon**: Geographical longitude (-180 to +180)
    - **house_system**: House system to use (default: Placidus)
    - **orb**: Maximum orb for aspects in degrees (default: 8.0)
    - **include_planets**: List of planets to include in chart (optional)
    """
    try:
        # Use all planets if not specified
        if include_planets is None:
            include_planets = list(PlanetType)
        
        # Parse date and time
        year, month, day, hour = parse_datetime(date, time)
        
        # Convert to Julian day
        jd = julian_day_ut(year, month, day, hour)
        
        # Calculate planet positions
        positions = {}
        for planet in include_planets:
            planet_id = get_planet_id(planet)
            try:
                position = calculate_planet_position(jd, planet_id)
                positions[planet] = position
            except Exception as e:
                logger.warning(f"Could not calculate position for planet {planet.value}: {str(e)}")
        
        # Calculate houses
        houses = calculate_houses(jd, geo_lat, geo_lon, house_system.value)
        
        # Calculate aspects
        aspects = calculate_aspects(positions, orb)
        
        # Format planet positions for the response
        planets_dict = {planet.value: PlanetPosition(**pos) for planet, pos in positions.items()}
        
        # Create and return the natal chart
        return NatalChart(
            planets=planets_dict,
            houses=HouseCusps(**houses),
            aspects=aspects,
            date=date,
            time=time,
            latitude=geo_lat,
            longitude=geo_lon
        )
    except Exception as e:
        logger.error(f"Error calculating natal chart: {str(e)}")
        raise HTTPException(status_code=400, detail=str(e))

@app.get("/health", tags=["System"])
async def health_check():
    """
    Health check endpoint to verify the API is running properly
    """
    try:
        # Try a simple calculation to check if Swiss Ephemeris is working
        jd = swe.julday(2000, 1, 1, 12)
        xx, ret = swe.calc_ut(jd, swe.SUN)
        
        return {
            "status": "ok",
            "swiss_ephemeris": "ok",
            "version": app.version,
            "ephemeris_path": os.environ.get("EPHE_PATH", "Not set"),
            "system_info": {
                "python_version": platform.python_version(),
                "platform": platform.platform(),
                "timezone": datetime.datetime.now().astimezone().tzinfo.tzname(None),
            }
        }
    except Exception as e:
        logger.error(f"Health check failed: {str(e)}")
        return {
            "status": "error",
            "message": str(e),
            "version": app.version
        }

@app.get("/info", tags=["System"])
async def system_info():
    """
    Get detailed information about the system
    """
    return {
        "api_version": app.version,
        "ephemeris_path": os.environ.get("EPHE_PATH", "Not set"),
        "system_info": {
            "python_version": platform.python_version(),
            "platform": platform.platform(),
            "timezone": datetime.datetime.now().astimezone().tzinfo.tzname(None),
        },
        "available_planets": [p.value for p in PlanetType],
        "available_house_systems": [h.value for h in HouseSystem],
        "available_aspect_types": [a.value for a in AspectType],
    }

#--------------------------------------------------------------------------#
# Main Entry Point                                                         #
#--------------------------------------------------------------------------#

def run_server():
    """Run the API server"""
    uvicorn.run(
        "astro_api_server:app",
        host=args.host,
        port=args.port,
        reload=False
    )

if __name__ == "__main__":
    # Run the server
    print(f"\nStarting Swiss Ephemeris API Server on {args.host}:{args.port}")
    print(f"API documentation available at: http://{args.host if args.host != '0.0.0.0' else 'localhost'}:{args.port}/docs")
    print(f"API Key: {API_KEY}")
    
    # If running directly (not imported), start the server
    run_server()
