import swisseph as swe
import os
import sys

def test_swiss_ephemeris():
    # Get the path from environment variable or use default
    ephe_path = r"C:\Users\hunta\Documents\GitHub\swisseph\ephe"
    
    print(f"Testing Swiss Ephemeris with path: {ephe_path}")
    
    # Set the ephemeris path
    swe.set_ephe_path(ephe_path)
    
    # Get current file data (includes internal paths)
    file_data = swe.get_current_file_data()
    print(f"Current file data: {file_data}")
    
    # Calculate the Sun's position
    jd = swe.julday(2025, 5, 13, 12.0)
    print(f"Julian day for May 13, 2025 12:00 UTC: {jd}")
    
    sun_data = None
    try:
        sun_data, flags = swe.calc_ut(jd, swe.SUN)
        print(f"Sun position: longitude={sun_data[0]}, latitude={sun_data[1]}, distance={sun_data[2]}")
        print(f"Calculation flags: {flags}")
        print("✓ Planetary calculations working correctly")
    except Exception as e:
        print(f"❌ Error calculating Sun position: {e}")
        sys.exit(1)
    
    # Try calculating house cusps
    try:
        houses, ascmc = swe.houses(jd, 47.0, 8.0, b'P')
        print(f"House cusps: {houses}")
        print(f"Asc/MC: {ascmc}")
        print("✓ House calculations working correctly")
    except Exception as e:
        print(f"❌ Error calculating houses: {e}")
    
    # Check for main asteroid files
    try:
        pos, flags = swe.calc_ut(jd, swe.CERES)
        print(f"Ceres position: longitude={pos[0]}")
        print("✓ Main asteroid calculations working")
    except Exception as e:
        print(f"❌ Error calculating Ceres position: {e}")
    
    # Check if asteroid directories exist
    asteroid_dirs_exist = False
    for i in range(10):
        dir_path = os.path.join(ephe_path, f"ast{i}")
        if os.path.isdir(dir_path):
            asteroid_dirs_exist = True
            print(f"✓ Found asteroid directory: ast{i}")
            # Count asteroid files
            files = [f for f in os.listdir(dir_path) if f.endswith('.se1')]
            print(f"  Contains {len(files)} asteroid files")
    
    if not asteroid_dirs_exist:
        print("⚠️ No asteroid directories found")
        print("To use asteroid calculations, create directories 'ast0', 'ast1', etc. in your ephemeris path")
    
    # Check for JPL files
    jpl_files = [
        "de200.eph", 
        "de406.eph", 
        "de431.eph", 
        "de441.eph"
    ]
    
    jpl_file_found = False
    for jpl_file in jpl_files:
        file_path = os.path.join(ephe_path, jpl_file)
        if os.path.isfile(file_path):
            jpl_file_found = True
            print(f"✓ Found JPL file: {jpl_file}")
    
    if not jpl_file_found:
        print("⚠️ No JPL files found. Using compressed files.")
        print("For highest precision calculations, consider downloading JPL ephemeris files.")
    
    # Test Successful
    print("\nSummary:")
    print("✓ Swiss Ephemeris configuration is working correctly")
    print("✓ Planetary calculations successful")
    print("✓ House calculations successful")
    
    if not asteroid_dirs_exist:
        print("⚠️ Asteroid directories missing (required for extended asteroid calculations)")
    
    if not jpl_file_found:
        print("⚠️ JPL files missing (required for highest precision calculations)")
    
    print("\nRecommendations:")
    if not asteroid_dirs_exist:
        print("1. Create asteroid directories (ast0, ast1, etc.) if you need extended asteroid calculations")
    if not jpl_file_found:
        print("2. Download JPL files if you need highest precision calculations")

if __name__ == "__main__":
    test_swiss_ephemeris()
